import xbmc, xbmcgui
from resources.lib.utils import *
from resources.lib.gui import *
from resources.lib.connection import Connection

class XBMCPlayer(xbmc.Player):
    """ xbmc player class """
    def __init__(self, *args, **kwargs):
        xbmc.Player.__init__(self)
        self.__playerPause = False
        self.__observers = []

    def __del__(self):
        del self.xbmc.Player

    def register_observer(self, observer):
        self.__observers.append(observer)

    def notify_observers(self, *args, **kwargs):
        for observer in self.__observers:
            observer.notify(self, *args, **kwargs)

    def notify(self):
        self.notify_observers("updateState")

    def isPausing(self):
        return self.__playerPause

    def onPlayBackPaused(self):
        self.__playerPause = True
        self.notify()

    def onPlayBackResumed(self):
        self.__playerPause = False
        self.notify()

    def onPlayBackStarted(self):
        self.notify()

    def onPlayBackStopped(self):
        self.__playerPause = False
        self.notify()

    def onPlayBackEnded(self):
        self.notify()

class XBMCMonitor(xbmc.Monitor):
    """ xbmc monitor class """
    def __init__(self, *args, **kwargs):
        xbmc.Monitor.__init__(self)
        self.__screensaverActive = False
        self.__observers = []

    def __del__(self):
        del self.xbmc.Monitor

    def register_observer(self, observer):
        self.__observers.append(observer)

    def notify_observers(self, *args, **kwargs):
        for observer in self.__observers:
            observer.notify(self, *args, **kwargs)

    def onSettingsChanged(self):
        self.notify_observers("updateSettings")

    def isScreensaverActive(self):
        return self.__screensaverActive

    def onScreensaverActivated(self):
        self.__screensaverActive = True
        self.notify_observers("updateState")

    def onScreensaverDeactivated(self):
        self.__screensaverActive = False
        self.notify_observers("updateState")

class Hyperion:
    """ Main instance class """
    def __init__(self):
        self.prev_videoMode = "2D"
        self.prev_compState = None
        self.initialized = False

        self.player = XBMCPlayer()
        self.player.register_observer(self)
        self.monitor = XBMCMonitor()
        self.monitor.register_observer(self)
        self.connection = Connection()

        self.updateSettings()
        self.daemon()

    def __del__(self):
        del self.player
        del self.monitor

    def notify(self, observable, *args, **kwargs):
        if "updateState" in args:
            self.updateState()
        else:
            self.updateSettings()

    def initialize(self):
        if getBoolSetting('firstRun'):
            doInitialWizard()
            setSetting('firstRun', 'False')

        if self.enableHyperion:
            self.connection.sendComponentState("ALL", True)

        self.initialized = True

    def updateSettings(self):
        # update settings, update the connection and updateState().
        self.ip = getSetting('ip')
        self.port = getSetting('port')
        self.videoModeEnabled = getBoolSetting('videoModeEnabled')
        self.enableHyperion = getBoolSetting('enableHyperion')
        self.disableHyperion = getBoolSetting('disableHyperion')
        self.opt_targetComp = intToCompString(int(getSetting('targetComponent')))
        self.opt_screensaverEnabled = getBoolSetting('screensaverEnabled')
        self.opt_videoEnabled = getBoolSetting('videoEnabled')
        self.opt_audioEnabled = getBoolSetting('audioEnabled')
        self.opt_pauseEnabled = getBoolSetting('pauseEnabled')
        self.opt_menuEnabled = getBoolSetting('menuEnabled')

        log('settings updated!')
        log('Hyperion ip:           %s' % (self.ip))
        log('Hyperion port:         %s' % (self.port))
        log('Enable H on start:     %s' % (self.enableHyperion))
        log('Disable H on stop:     %s' % (self.disableHyperion))
        log('VideoMode enabled:     %s' % (self.videoModeEnabled))
        log('Hyperion target comp:  %s' % (self.opt_targetComp))
        log('Screensaver enabled:   %s' % (self.opt_screensaverEnabled))
        log('Video enabled:         %s' % (self.opt_videoEnabled))
        log('Audio enabled:         %s' % (self.opt_audioEnabled))
        log('Pause enabled:         %s' % (self.opt_pauseEnabled))
        log('Menu enabled:          %s' % (self.opt_menuEnabled))

        self.connection.updateURL(self.ip, self.port)

        # do just once on startup
        if not self.initialized:
            self.initialize()

        self.updateState()

    def daemon(self):
        # Keep the Hyperion class alive, aborts if Kodi requests it
        while not self.monitor.abortRequested():
            if self.monitor.waitForAbort():
                # last steps before script shutdown
                if self.disableHyperion:
                    self.connection.sendComponentState("ALL", False)

                log("Hyperion-control stopped")
                break

    def updateState(self):
        # update state of the chosen component based on latest settings/kodi states and send the state to hyperion
        # check first screensaver, then pause and as last player!
        if self.monitor.isScreensaverActive():
                log("SCREENSAVER MODE")
                compState = True if self.opt_screensaverEnabled else False
        elif self.player.isPausing():
                log("PAUSE MODE")
                compState = True if self.opt_pauseEnabled else False
        elif self.player.isPlaying():
            if self.player.isPlayingVideo():
                log("VIDEO PLAYING")
                compState = True if self.opt_videoEnabled else False
            elif self.player.isPlayingAudio():
                log("AUDIO PLAYING")
                compState = True if self.opt_audioEnabled else False
        else:
            log("MENU MODE")
            compState = True if self.opt_menuEnabled else False

        # update comp state just if required
        if self.prev_compState != compState:
            self.connection.sendComponentState(self.opt_targetComp, compState)
            self.prev_compState = compState

        # update stereoscrope mode always, better apis for detection available?
        # Bug: race condition, return of jsonapi has wrong gui state after onPlayBackStopped after a 3dmovie
        if self.videoModeEnabled:
            newMode = getStereoscopeMode()
            if self.prev_videoMode != newMode:
                self.connection.sendVideoMode(newMode)
                self.prev_videoMode = newMode

if __name__ == '__main__':
    hyperion = Hyperion()
    del hyperion
